@@ -20,6 +20,7 @@
 
 
 #include <ctype.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -35,9 +36,10 @@
 struct gpio_list {
 	struct gpio_list *next;
 	int gpio;
+	int input; /* 1 if direction of gpio should be changed back to input. */
+	int exported; /* 0 if gpio should be unexported. */
 };
 
-
 static int write_to(const char *filename, const char *value)
 {
 	int fd, ret;
@@ -64,15 +66,45 @@ static int drive_gpio(int n, int level, struct gpio_list **gpio_to_release)
 	return 0;
 }
 #else
+static int read_from(const char *filename, char *buf, size_t len)
+{
+	int fd, ret;
+	ssize_t n = 0;
+
+	fd = open(filename, O_RDONLY);
+	if (fd < 0) {
+		fprintf(stderr, "Cannot open file \"%s\"\n", filename);
+		return 0;
+	}
+
+	do {
+		ret = read(fd, buf + n, len - n);
+		if (ret < 0) {
+			if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)
+				continue; /* try again */
+			fprintf(stderr, "Error reading in file \"%s\"\n", filename);
+			close(fd);
+			return 0;
+		}
+		n += ret;
+	} while (n < len && ret);
+
+	close(fd);
+	return n;
+}
+
 static int drive_gpio(int n, int level, struct gpio_list **gpio_to_release)
 {
 	char num[16]; /* sized to carry MAX_INT */
 	char file[48]; /* sized to carry longest filename */
+	char dir;
 	struct stat buf;
 	struct gpio_list *new;
 	int ret;
+	int exported = 1;
+	int input = 0;
 
-	sprintf(file, "/sys/class/gpio/gpio%d/direction", n);
+	sprintf(file, "/sys/class/gpio/gpio%d/value", n);
 	ret = stat(file, &buf);
 	if (ret) {
 		/* file miss, GPIO not exported yet */
@@ -85,12 +117,25 @@ static int drive_gpio(int n, int level, struct gpio_list **gpio_to_release)
 			fprintf(stderr, "GPIO %d not available\n", n);
 			return 0;
 		}
+		exported = 0;
+	}
+
+	sprintf(file, "/sys/class/gpio/gpio%d/direction", n);
+	ret = stat(file, &buf);
+	if (!ret)
+		if (read_from(file, &dir, sizeof(dir)))
+			if (dir == 'i')
+				input = 1;
+
+	if (exported == 0 || input == 1) {
 		new = (struct gpio_list *)malloc(sizeof(struct gpio_list));
 		if (new == NULL) {
 			fprintf(stderr, "Out of memory\n");
 			return 0;
 		}
 		new->gpio = n;
+		new->exported = exported;
+		new->input = input;
 		new->next = *gpio_to_release;
 		*gpio_to_release = new;
 	}
@@ -99,12 +144,20 @@ static int drive_gpio(int n, int level, struct gpio_list **gpio_to_release)
 }
 #endif
 
-static int release_gpio(int n)
+static int release_gpio(int n, int input, int exported)
 {
 	char num[16]; /* sized to carry MAX_INT */
+	char file[48]; /* sized to carry longest filename */
 
 	sprintf(num, "%d", n);
-	return write_to("/sys/class/gpio/unexport", num);
+	if (input) {
+		sprintf(file, "/sys/class/gpio/gpio%d/direction", n);
+		write_to(file, "in");
+	}
+	if (!exported)
+		write_to("/sys/class/gpio/unexport", num);
+
+	return 1;
 }
 
 static int gpio_sequence(struct port_interface *port, const char *s, size_t l)
@@ -163,7 +216,7 @@ static int gpio_sequence(struct port_interface *port, const char *s, size_t l)
 	}
 
 	while (gpio_to_release) {
-		release_gpio(gpio_to_release->gpio);
+		release_gpio(gpio_to_release->gpio, gpio_to_release->input, gpio_to_release->exported);
 		to_free = gpio_to_release;
 		gpio_to_release = gpio_to_release->next;
 		free(to_free);
@@ -210,7 +263,7 @@ int init_bl_entry(struct port_interface *port, const char *seq)
 
 int init_bl_exit(stm32_t *stm, struct port_interface *port, const char *seq)
 {
-	if (seq)
+	if (seq && strchr(seq, ':'))
 		return gpio_bl_exit(port, seq);
 
 	if (stm32_reset_device(stm) != STM32_ERR_OK)
