@@ -18,6 +18,7 @@
 */
 
 #include <fcntl.h>
+#include <limits.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,6 +26,7 @@
 #include <termios.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
+#include <stdio.h>
 
 #include "serial.h"
 #include "port.h"
@@ -202,11 +204,7 @@ static port_err_t serial_posix_open(struct port_interface *port,
 {
 	serial_t *h;
 
-	/* 1. check device name match */
-	if (strncmp(ops->device, "/dev/tty", strlen("/dev/tty")))
-		return PORT_ERR_NODEV;
-
-	/* 2. check options */
+	/* 1. check options */
 	if (ops->baudRate == SERIAL_BAUD_INVALID)
 		return PORT_ERR_UNKNOWN;
 	if (serial_get_bits(ops->serial_mode) == SERIAL_BITS_INVALID)
@@ -216,11 +214,15 @@ static port_err_t serial_posix_open(struct port_interface *port,
 	if (serial_get_stopbit(ops->serial_mode) == SERIAL_STOPBIT_INVALID)
 		return PORT_ERR_UNKNOWN;
 
-	/* 3. open it */
+	/* 2. open it */
 	h = serial_open(ops->device);
 	if (h == NULL)
 		return PORT_ERR_UNKNOWN;
 
+	/* 3. check for tty (but only warn) */
+	if (!isatty(h->fd))
+		fprintf(stderr, "Warning: Not a tty: %s\n", ops->device);
+
 	/* 4. set options */
 	if (serial_setup(h, ops->baudRate,
 			 serial_get_bits(ops->serial_mode),
