@@ -67,6 +67,7 @@ static serial_t *serial_open(const char *device)
 	if (h->fd == INVALID_HANDLE_VALUE) {
 		if (GetLastError() == ERROR_FILE_NOT_FOUND)
 			fprintf(stderr, "File not found: %s\n", device);
+		free(h);
 		return NULL;
 	}
 
@@ -76,6 +77,9 @@ static serial_t *serial_open(const char *device)
 
 	SetCommMask(h->fd, EV_ERR); /* Notify us of error events */
 
+	/* DCBlength should be initialized before calling GetCommState */
+	h->oldtio.DCBlength = sizeof(DCB);
+	h->newtio.DCBlength = sizeof(DCB);
 	GetCommState(h->fd, &h->oldtio); /* Retrieve port parameters */
 	GetCommState(h->fd, &h->newtio); /* Retrieve port parameters */
 
@@ -161,10 +165,15 @@ static port_err_t serial_setup(serial_t *h,
 	/* reset the settings */
 	h->newtio.fOutxCtsFlow = FALSE;
 	h->newtio.fOutxDsrFlow = FALSE;
+	h->newtio.fDtrControl = DTR_CONTROL_DISABLE;
+	h->newtio.fDsrSensitivity = FALSE;
+	h->newtio.fTXContinueOnXoff = FALSE;
 	h->newtio.fOutX = FALSE;
 	h->newtio.fInX = FALSE;
-	h->newtio.fNull = 0;
-	h->newtio.fAbortOnError = 0;
+	h->newtio.fErrorChar = FALSE;
+	h->newtio.fNull = FALSE;
+	h->newtio.fRtsControl = RTS_CONTROL_DISABLE;
+	h->newtio.fAbortOnError = FALSE;
 
 	/* set the settings */
 	serial_flush(h);
@@ -186,8 +195,7 @@ static port_err_t serial_w32_open(struct port_interface *port,
 	serial_t *h;
 
 	/* 1. check device name match */
-	if (!(strlen(ops->device) == 4
-	      && !strncmp(ops->device, "COM", 3) && isdigit(ops->device[3]))
+	if (!(!strncmp(ops->device, "COM", 3) && isdigit(ops->device[3]))
 	    && !(!strncmp(ops->device, "\\\\.\\COM", strlen("\\\\.\\COM"))
 		 && isdigit(ops->device[strlen("\\\\.\\COM")])))
 		return PORT_ERR_NODEV;
