@@ -27,6 +27,9 @@
 #define STM32_MAX_RX_FRAME	256	/* cmd read memory */
 #define STM32_MAX_TX_FRAME	(1 + 256 + 1)	/* cmd write memory */
 
+#define STM32_MAX_PAGES		0x0000ffff
+#define STM32_MASS_ERASE	0x00100000 /* > 2 x max_pages */
+
 typedef enum {
 	STM32_ERR_OK = 0,
 	STM32_ERR_UNKNOWN,	/* Generic error */
@@ -34,6 +37,11 @@ typedef enum {
 	STM32_ERR_NO_CMD,	/* Command not available in bootloader */
 } stm32_err_t;
 
+typedef enum {
+	F_NO_ME = 1 << 0,	/* Mass-Erase not supported */
+	F_OBLL  = 1 << 1,	/* OBL_LAUNCH required */
+} flags_t;
+
 typedef struct stm32		stm32_t;
 typedef struct stm32_cmd	stm32_cmd_t;
 typedef struct stm32_dev	stm32_dev_t;
@@ -55,9 +63,10 @@ struct stm32_dev {
 	uint32_t	ram_start, ram_end;
 	uint32_t	fl_start, fl_end;
 	uint16_t	fl_pps; // pages per sector
-	uint16_t	fl_ps;  // page size
+	uint32_t	*fl_ps;  // page size
 	uint32_t	opt_start, opt_end;
 	uint32_t	mem_start, mem_end;
+	uint32_t	flags;
 };
 
 stm32_t *stm32_init(struct port_interface *port, const char init);
@@ -68,8 +77,8 @@ stm32_err_t stm32_write_memory(const stm32_t *stm, uint32_t address,
 			       const uint8_t data[], unsigned int len);
 stm32_err_t stm32_wunprot_memory(const stm32_t *stm);
 stm32_err_t stm32_wprot_memory(const stm32_t *stm);
-stm32_err_t stm32_erase_memory(const stm32_t *stm, uint8_t spage,
-			       uint8_t pages);
+stm32_err_t stm32_erase_memory(const stm32_t *stm, uint32_t spage,
+			       uint32_t pages);
 stm32_err_t stm32_go(const stm32_t *stm, uint32_t address);
 stm32_err_t stm32_reset_device(const stm32_t *stm);
 stm32_err_t stm32_readprot_memory(const stm32_t *stm);
